import json
import click
import sys
from click_option_group import optgroup, RequiredMutuallyExclusiveOptionGroup
from cli.utils import exception_handler, wowza_auth
from cli.database import Database
from wowpy.resources import get_resource_info
from pygments import highlight
from pygments.lexers import JsonLexer
from pygments.formatters import TerminalTrueColorFormatter

PROFILE_HELP = """
`wowtool` profile, can be set with WOWTOOL_PROFILE env var.
profile settings in ~/.wow/config.json
"""


@click.group()
def resource():
    """
    Resource commands.
    """
    pass

@resource.command(name='query')
@optgroup.group('Resource identifier', cls=RequiredMutuallyExclusiveOptionGroup,
                help='Identifier for resource')
@optgroup.option('--id', help='Resource stream id')
@optgroup.option('--name', help='Resource stream name')
@exception_handler
def query(id, name):
    """Get data for a resource"""
    wowza_auth()
    if name:
        database = Database()
        if not database.live_streams_table_exist():
            click.echo('In order to use --name parameter please run: wowtool mapper live_streams, first')
            sys.exit()

        id = database.get_live_stream_id_from_name(name)
        if not id:
            click.echo('Live stream not found in current mapping table')
            sys.exit()
    response = get_resource_info(id)
    formatted_json = json.dumps(response, indent=4)
    colorful_json = highlight(formatted_json, JsonLexer(), TerminalTrueColorFormatter(style='solarized-dark'))
    click.echo(colorful_json)


from cli.utils import get_base_file_path
from tinydb import TinyDB, Query

class Database:

  # TODO: extract this to a config file
  database_filename = 'database.json'

  def __init__(self):
    database_file_path = get_base_file_path(filename=self.database_filename)
    self.db = TinyDB(database_file_path)

  def live_streams_table_exist(self):
    table_name = 'live_streams'
    tables = self.db.tables()
    exist = table_name in tables
    return exist

  def get_live_stream_id_from_name(self, name):
    id = None
    table_name = 'live_streams'
    query = Query()
    table = self.db.table(table_name)
    response = table.search(query.live_stream_name == name)
    if response:
      id = response[0].get('live_stream_id', None)
    return id

  def recordings_table_exist(self):
    table_name = 'recordings'
    tables = self.db.tables()
    exist = table_name in tables
    return exist

  def get_recording_id_from_name(self, name):
    id = None
    table_name = 'recordings'
    query = Query()
    table = self.db.table(table_name)
    response = table.search(query.live_stream_name == name)
    if response:
      id = response[0].get('recording_id', None)
    return id

  def insert_live_streams_document(self, document_batch):
    table_name = 'live_streams'
    self.db.drop_table(table_name)
    table = self.db.table(table_name)
    table.insert_multiple(document_batch)

  def insert_recordings_document(self, document_batch):
    table_name = 'recordings'
    self.db.drop_table(table_name)
    table = self.db.table(table_name)
    table.insert_multiple(document_batch)



from cli.database import Database

# TODO: Load data into database

def test_live_streams_table_exist():
    database = Database()
    response = database.live_streams_table_exist()
    assert response in [False, True]

# def get_live_stream_id_from_name():
#     pass


import functools
import click
import datetime
import json
import os
from pathlib import Path

def exception_handler(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            message = 'There was an error in the CLI: {}'.format(str(e))
            click.echo(message, err=True)
            exit(1)

    return wrapper

# core auth

def wowza_auth():
    if config_file_exist():
        __set_credentials_envars()
    else:
        __create_creds_file()
        __set_credentials_envars()

def __set_credentials_envars():
    credentials_file_path = get_base_file_path()
    with open(credentials_file_path) as json_file:
        credentials_data = json.load(json_file)
        os.environ["WSC_ACCESS_KEY"] = credentials_data['WSC_ACCESS_KEY']
        os.environ["WSC_API_KEY"] = credentials_data['WSC_API_KEY']

def __create_creds_file():
    config_path=__get_config_path()
    wsc_access_key = click.prompt('Please enter wowza access key')
    wsc_api_key = click.prompt('Please enter wowza api key')
    keys = {
            'WSC_ACCESS_KEY': wsc_access_key,
            'WSC_API_KEY': wsc_api_key
            }
    Path(config_path).mkdir(parents=True, exist_ok=True)
    credentials_file_path = get_base_file_path()
    with open(credentials_file_path, 'w') as json_file:
        json_file.write(json.dumps(keys, indent=4))

# core paths

def __get_config_path():
    home_path = str(Path.home())
    config_path = home_path + '/.wow/'
    return config_path

def __get_config_filename():
    config_filename = 'config.json'
    return config_filename

def config_file_exist(config_path=__get_config_path(), filename=__get_config_filename()):
    base_file_path = get_base_file_path(config_path, filename=filename)
    if os.path.exists(base_file_path):
        return True
    else:
        return False

def get_base_file_path(config_path=__get_config_path(), filename=__get_config_filename()):
    full_file_path = config_path + filename
    return full_file_path



import logging
from logging import Logger
import click
from cli.__version__ import VERSION
from cli.commands.livestream import live
from cli.commands.transcoder import trans
from cli.commands.configure import configure
from cli.commands.resource import resource 
from cli.commands.recording import recording
from cli.commands.mapping import mapper


@click.group()
@click.version_option(version=VERSION)
def cli():
    pass

cli.add_command(mapper)
cli.add_command(resource)
cli.add_command(live)
cli.add_command(trans)
cli.add_command(recording)
cli.add_command(configure)

# Logger setup
loglevel = 'ERROR'
logger = Logger('')
logger.setLevel(loglevel)
ch = logging.StreamHandler()
ch.setLevel(loglevel)
logger.addHandler(ch)


import json
import sys
import click
from tinydb import TinyDB, Query
from click_option_group import optgroup, RequiredMutuallyExclusiveOptionGroup
from cli.utils import exception_handler, wowza_auth
from cli.database import Database
from pygments import highlight
from pygments.lexers import JsonLexer
from pygments.formatters import TerminalTrueColorFormatter
from wowpy.recordings import Recording

PROFILE_HELP = """
`wowtool` profile, can be set with WOWTOOL_PROFILE env var.
profile settings in ~/.wow/config.json
"""


@click.group()
def recording():
    """
    Recording commands.
    """
    pass


@recording.command(name='query')
@optgroup.group('Recording identifier', cls=RequiredMutuallyExclusiveOptionGroup,
                help='Identifier for wowza recording')
@optgroup.option('--id', help='Live stream id')
@optgroup.option('--name', help='Live stream name')
@exception_handler
def query(id, name):
    """Get Recording"""
    wowza_auth()
    if name:
        database = Database()
        if not database.recordings_table_exist():
            click.echo('In order to use --name parameter please run: wowtool mapper recordings, first')
            sys.exit()

        id = database.get_recording_id_from_name(name)
        if not id:
            click.echo('Recording not found in current mapping table')
            sys.exit()
    response = Recording.get_recording(id)
    formatted_json = json.dumps(response, indent=4)
    colorful_json = highlight(formatted_json, JsonLexer(), TerminalTrueColorFormatter(style='solarized-dark'))
    click.echo(colorful_json)
